# coding:utf-8

import requests
import re
import json
from bs4 import BeautifulSoup


class youdao_web:
    web_url = 'http://dict.youdao.com/search?keyfrom=dict.top&q='
    translation_url = 'http://fanyi.youdao.com/translate?keyfrom=dict.top&i='
    result = {
        "query": "",
        "errorCode": 0,
    }

    def get_result(self, word):
        self.word = word
        r = requests.get(self.web_url + word)
        r.raise_for_status()
        self.parse(r.text)
        return self.result

    def parse(self, html):
        soup = BeautifulSoup(html)
        root = soup.find(id='results-contents')

        #query 搜索的关键字
        keyword = root.find(class_='keyword')
        if not keyword:
            self.result['query'] = self.word
        else:
            self.result['query'] = unicode(keyword.string)

        #基本解释
        basic = root.find(id='phrsListTab')
        if basic:
            trans = basic.find(class_='trans-container')
            if trans:
                self.result['basic'] = {}
                self.result['basic']['explains'] = [unicode(tran.string) for tran in trans.find_all('li')]
                #中文
                if len(self.result['basic']['explains']) == 0:
                    exp = trans.find(class_='wordGroup').stripped_strings
                    self.result['basic']['explains'].append(' '.join(exp))

                #音标
                phons = basic(class_='phonetic', limit=2)
                if len(phons) == 2:
                    self.result['basic']['uk-phonetic'], self.result['basic']['us-phonetic'] = \
                        [unicode(p.string)[1:-1] for p in phons]
                elif len(phons) == 1:
                    self.result['basic']['phonetic'] = unicode(phons[0].string)[1:-1]

        #翻译
        if 'basic' not in self.result:
            self.result['translation'] = self.get_translation(self.word)


        #网络释义(短语)
        web = root.find(id='webPhrase')
        if web:
            self.result['web'] = [
                {
                    'key': unicode(wordgroup.find(class_='search-js').string).strip(),
                    'value': [v.strip() for v in unicode(wordgroup.find('span').next_sibling).split(';')]
                } for wordgroup in web.find_all(class_='wordGroup', limit=4)
            ]
            # for wordgroup in web.find_all(class_='wordGroup', limit=4):
            #     item = {'key': unicode(wordgroup.find(class_='search-js').string).strip(),
            #             'value': [v.strip() for v in unicode(wordgroup.find('span').next_sibling).split(';')]}
            #     self.result['web'].append(item)

    def get_translation(self, word):
        """
        通过web版有道翻译抓取翻译结果
        :param word:str 关键字
        :return:list 翻译结果
        """
        r = requests.get(self.translation_url+word)
        if r.status_code != requests.codes.ok:
            return None

        pattern = re.compile(r'"translateResult":\[(\[.+\])\]')
        m = pattern.search(r.text)
        result = json.loads(m.group(1))
        return [item['tgt'] for item in result]


if __name__ == '__main__':
    test = youdao_web()
    print test.get_result('application')


# coding: utf-8

from peewee import *
import json
import os

HOME = os.path.expanduser("~")
db_dir = os.path.join(HOME, '.dict_youdao')
if not os.path.exists(db_dir):
    os.mkdir(db_dir)
db = SqliteDatabase(os.path.join(db_dir, 'youdao.db'))


class BaseModel(Model):
    class Meta:
        database = db


class Word(BaseModel):
    word = CharField(index=True, unique=True)
    json_data = TextField()


class youdao_db:
    def __init__(self):
        db.create_tables([Word], safe=True)

    def save_word(self, aword, dict_data):
        wid = self.get_word_id(aword)

        word = Word()
        word.id = wid
        word.word = aword
        word.json_data = json.dumps(dict_data)
        word.save()

    def get_word(self, word):
        try:
            word = Word.select(Word.json_data).where(Word.word == word).get()
            return word.json_data
        except Word.DoesNotExist:
            return None

    def get_word_id(self, word):
        try:
            word = Word.select(Word.id).where(Word.word == word).get()
            return word.id
        except Word.DoesNotExist:
            return None

    def get_all_word(self):
        return [row.word for row in Word.select(Word.word)]

    def del_word(self, word=None):
        if word:
            Word.delete().where(Word.word == word).execute()
        else:
            Word.delete().execute()

    def __del__(self):
        db.close()


# coding:utf-8

from setuptools import setup, find_packages

version = '0.1.5'

setup(name='youdao',
      version=version,
      description="基于有道API和有道词典web版的在terminal查询词的小工具,支持单词发音",
      long_description=open('README.md').read(),
      keywords='python youdao dictionary terminal',
      author='longcw',
      author_email='longchsin@foxmail.com',
      url='https://github.com/longcw/youdao',
      license='',
      packages=find_packages(exclude=['tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'termcolor', 'requests', 'beautifulsoup4', 'peewee'
      ],
      entry_points={
          'console_scripts': [
              'yd = youdao.youdao:main',
          ]
      },
)



# coding:utf-8

import sys
import os
import getopt
import requests
import json
import webbrowser

from termcolor import colored
from youdao_web import youdao_web
from youdao_db import youdao_db, db_dir


class youdao:
    params = {
        'keyfrom': 'longcwang',
        'key': '131895274',
        'type': 'data',
        'doctype': 'json',
        'version': '1.1',
        'q': 'query'
    }
    api_url = 'http://fanyi.youdao.com/openapi.do'
    voice_url = 'http://dict.youdao.com/dictvoice?type=2&audio={word}'

    error_code = {
        0: u'正常',
        20: u'要翻译的文本过长',
        30: u'无法进行有效的翻译',
        40: u'不支持的语言类型',
        50: u'无效的key',
        60: u'无词典结果，仅在获取词典结果生效'
    }

    def get_voice(self, word):
        print(colored(u'获取发音中:{word}'.format(word=word), 'green'))
        voice_dir = os.path.join(db_dir, 'voice')
        if not os.path.exists(voice_dir):
            os.mkdir(voice_dir)

        voice_file = os.path.join(voice_dir, word+'.mp3')
        if not os.path.isfile(voice_file):
            r = requests.get(self.voice_url.format(word=word))
            with open(voice_file, 'wb') as f:
                f.write(r.content)
        print(colored(u'获取成功,播放中...', 'green'))

        saveout1 = os.dup(1)
        saveout2 = os.dup(2)
        os.close(1)
        os.close(2)
        try:
            webbrowser.open(voice_file)
        finally:
            os.dup2(saveout1, 1)
            os.dup2(saveout2, 2)

    def get_response(self, word, use_api):
        if use_api:
            self.params['q'] = word
            r = requests.get(self.api_url, params=self.params)
            r.raise_for_status()    # a 4XX client error or 5XX server error response
            result = r.json()
        else:
            yd = youdao_web()
            result = yd.get_result(word)

        return result

    def show(self, result):
        if result['errorCode'] != 0:
            print colored(self.error_code[result['errorCode']], 'red')
        else:
            print colored('[%s]' % result['query'], 'magenta')
            if 'basic' in result:
                if 'us-phonetic' in result['basic']:
                    print colored(u'美音:', 'blue'), colored('[%s]' % result['basic']['us-phonetic'], 'green'),
                if 'uk-phonetic' in result['basic']:
                    print colored(u'英音:', 'blue'), colored('[%s]' % result['basic']['uk-phonetic'], 'green')
                if 'phonetic' in result['basic']:
                    print colored(u'拼音:', 'blue'), colored('[%s]' % result['basic']['phonetic'], 'green')

                print colored(u'基本词典:', 'blue')
                print colored('\t'+'\n\t'.join(result['basic']['explains']), 'yellow')

            if 'translation' in result:
                print colored(u'有道翻译:', 'blue')
                print colored('\t'+'\n\t'.join(result['translation']), 'cyan')

            if 'web' in result:
                print colored(u'网络释义:', 'blue')
                for item in result['web']:
                    print '\t' + colored(item['key'], 'cyan') + ': ' + '; '.join(item['value'])

    def query(self, word, use_db=True, use_api=False, play_voice=False):
        try:
            db = youdao_db()
            result = None
            if use_db:
                data = db.get_word(word)
                if data:
                    result = json.loads(data)
            if not result:
                result = self.get_response(word, use_api)
                db.save_word(word, result)
            self.show(result)
            if play_voice:
                self.get_voice(word)

        except requests.HTTPError as e:
            print colored(u'网络错误: %s' % e.message, 'red')


def show_db_list():
    db = youdao_db()
    words = db.get_all_word()
    print colored(u'保存的单词:', 'blue')
    print colored('\n'.join(words), 'cyan')


def del_word(word):
    db = youdao_db()
    db.del_word(word)


def show_help():
    print(u"""
    控制台下的有道词典 版本0.1.4
    默认通过解析有道网页版获取查询结果, 没有词典结果时自动使用有道翻译,
    查询结果会保存到sqlite 数据库中
    使用方法 yd word [-a] [-n] [-l] [-c] [-d word] [--help]
    [-a] 使用API 而不是解析网页获取结果
    [-n] 强制重新获取, 不管数据库中是否已经保存
    [-l] 列出数据库中保存的所有单词
    [-c] 清空数据库
    [-d word] 删除数据库中某个单词
    [--help] 显示帮助信息
    """)


def main():
    try:
        options, args = getopt.getopt(sys.argv[1:], 'anld:cv', ['help'])
    except getopt.GetoptError:
        options = [('--help', '')]
    if ('--help', '') in options:
        show_help()
        return

    yd = youdao()
    use_api = False
    use_db = True
    play_voice = False
    for opt in options:
        if opt[0] == '-a':
            use_api = True
        elif opt[0] == '-n':
            use_db = False
        elif opt[0] == '-l':
            show_db_list()
            return
        elif opt[0] == '-d':
            del_word(opt[1])
            return
        elif opt[0] == '-c':
            del_word(None)
            return
        elif opt[0] == '-v':
            play_voice = True

    word = ' '.join(args)

    while not word:
        word = raw_input(colored('input a word: ', 'blue'))

    yd.query(word, use_db, use_api, play_voice)

if __name__ == '__main__':
    main()